@@ -20,6 +20,7 @@ from jenkinsapi.result_set import ResultSet
 from jenkinsapi.jenkinsbase import JenkinsBase
 from jenkinsapi.constants import STATUS_SUCCESS
 from jenkinsapi.custom_exceptions import NoResults
+from jenkinsapi.custom_exceptions import JenkinsAPIException
 
 
 log = logging.getLogger(__name__)
@@ -33,16 +34,25 @@ class Build(JenkinsBase):
     STR_TOTALCOUNT = "totalCount"
     STR_TPL_NOTESTS_ERR = "%s has status %s, and does not have any test results"
 
-    def __init__(self, url, buildno, job):
+    def __init__(self, url, buildno, job, depth=1):
+        """
+        depth=1 is for backward compatibility consideration
+
+        About depth, the deeper it is, the more build data you get back. If
+        depth=0 is sufficient for you, don't go up to 1. See section 'Depth
+        control' of
+        https://wiki.jenkins-ci.org/display/JENKINS/Remote+access+API
+        """
         assert type(buildno) == int
         self.buildno = buildno
         self.job = job
+        self.depth = depth
         JenkinsBase.__init__(self, url)
 
     def _poll(self):
         # For build's we need more information for downstream and upstream builds
         # so we override the poll to get at the extra data for build objects
-        url = self.python_api_url(self.baseurl) + '?depth=1'
+        url = '%s?depth=%s' % (self.python_api_url(self.baseurl), self.depth)
         return self.get_data(url)
 
     def __str__(self):
@@ -67,7 +77,7 @@ class Build(JenkinsBase):
         return getattr(self, '_get_%s_rev_branch' % vcs, lambda: None)()
 
     def _get_svn_rev(self):
-        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
+        warnings.warn("This untested function may soon be removed from Jenkinsapi (get_svn_rev).")
         maxRevision = 0
         for repoPathSet in self._data["changeSet"]["revisions"]:
             maxRevision = max(repoPathSet["revision"], maxRevision)
@@ -82,7 +92,7 @@ class Build(JenkinsBase):
         return _actions[0]["lastBuiltRevision"]["SHA1"]
 
     def _get_hg_rev(self):
-        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
+        warnings.warn("This untested function may soon be removed from Jenkinsapi (_get_hg_rev).")
         return [x['mercurialNodeName'] for x in self._data['actions'] if 'mercurialNodeName' in x][0]
 
     def _get_svn_rev_branch(self):
@@ -159,7 +169,6 @@ class Build(JenkinsBase):
         Get the master job name if it exist, None otherwise
         :return: String or None
         """
-        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
         try:
             return self.get_actions()['parameters'][0]['value']
         except KeyError:
@@ -170,7 +179,7 @@ class Build(JenkinsBase):
         Get the master job object if it exist, None otherwise
         :return: Job or None
         """
-        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
+        warnings.warn("This untested function may soon be removed from Jenkinsapi (get_master_job).")
         if self.get_master_job_name():
             return self.get_jenkins_obj().get_job(self.get_master_job_name())
         else:
@@ -181,7 +190,7 @@ class Build(JenkinsBase):
         Get the master build number if it exist, None otherwise
         :return: int or None
         """
-        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
+        warnings.warn("This untested function may soon be removed from Jenkinsapi (get_master_build_number).")
         try:
             return int(self.get_actions()['parameters'][1]['value'])
         except KeyError:
@@ -192,7 +201,7 @@ class Build(JenkinsBase):
         Get the master build if it exist, None otherwise
         :return Build or None
         """
-        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
+        warnings.warn("This untested function may soon be removed from Jenkinsapi (get_master_build).")
         master_job = self.get_master_job()
         if master_job:
             return master_job.get_build(self.get_master_build_number())
@@ -204,7 +213,7 @@ class Build(JenkinsBase):
         Get the downstream jobs for this build
         :return List of jobs or None
         """
-        warnings.warn("This untested function may soon be removed from Jenkinsapi.")
+        warnings.warn("This untested function may soon be removed from Jenkinsapi (get_downstream_jobs).")
         downstream_jobs = []
         try:
             for job_name in self.get_downstream_job_names():
@@ -218,18 +227,6 @@ class Build(JenkinsBase):
         Get the downstream job names for this build
         :return List of string or None
         """
-# <<<<<<< HEAD
-#         downstream_jobs_names = self.job.get_downstream_job_names()
-#         fingerprint_data = self.get_data("%s?depth=2&tree=fingerprint[usage[name]]" \
-#                                            % self.python_api_url(self.baseurl))
-#         try:
-#             fingerprints = fingerprint_data['fingerprint'][0]
-#             return [
-#                 f['name']
-#                 for f in fingerprints['usage']
-#                 if f['name'] in downstream_jobs_names
-#             ]
-# =======
         downstream_job_names = self.job.get_downstream_job_names()
         downstream_names = []
         try:
@@ -239,7 +236,6 @@ class Build(JenkinsBase):
                     if job_usage['name'] in downstream_job_names:
                         downstream_names.append(job_usage['name'])
             return downstream_names
-# >>>>>>> unstable
         except (IndexError, KeyError):
             return []
 
@@ -248,18 +244,6 @@ class Build(JenkinsBase):
         Get the downstream builds for this build
         :return List of Build or None
         """
-# <<<<<<< HEAD
-#         downstream_jobs_names = set(self.job.get_downstream_job_names())
-#         msg = "%s?depth=2&tree=fingerprint[usage[name,ranges[ranges[end,start]]]]"
-#         fingerprint_data = self.get_data(msg % self.python_api_url(self.baseurl))
-#         try:
-#             fingerprints = fingerprint_data['fingerprint'][0]
-#             return [
-#                 self.get_jenkins_obj().get_job(f['name']).get_build(f['ranges']['ranges'][0]['start'])
-#                 for f in fingerprints['usage']
-#                 if f['name'] in downstream_jobs_names
-#             ]
-# =======
         downstream_job_names = self.get_downstream_job_names()
         downstream_builds = []
         try:
@@ -273,7 +257,6 @@ class Build(JenkinsBase):
                                                   job_range['end']):
                                 downstream_builds.append(job.get_build(build_id))
             return downstream_builds
-# >>>>>>> unstable
         except (IndexError, KeyError):
             return []
 
@@ -351,6 +334,21 @@ class Build(JenkinsBase):
             all_actions.update(dct_action)
         return all_actions
 
+    def get_causes(self):
+        '''
+        Returns a list of causes. There can be multiple causes lists and
+        some of the can be empty. For instance, when a build is manually
+        aborted, Jenkins could add an empty causes list to the actions
+        dict. Empty ones are ignored.
+        '''
+        all_causes = []
+        for dct_action in self._data["actions"]:
+            if dct_action is None:
+                continue
+            if 'causes' in dct_action and dct_action['causes']:
+                all_causes.extend(dct_action['causes'])
+        return all_causes
+
     def get_timestamp(self):
         '''
         Returns build timestamp in UTC
@@ -364,7 +362,16 @@ class Build(JenkinsBase):
         Return the current state of the text console.
         """
         url = "%s/consoleText" % self.baseurl
-        return self.job.jenkins.requester.get_url(url).content
+        content = self.job.jenkins.requester.get_url(url).content
+        # This check was made for Python 3.x
+        # In this version content is a bytes string
+        # By contract this function must return string
+        if isinstance(content, str):
+            return content
+        elif isinstance(content, bytes):
+            return content.decode('utf-8')
+        else:
+            raise JenkinsAPIException('Unknown content type for console')
 
     def stop(self):
         """
